class Provider():
    def get_kwarg(self, name, **kwargs):
        if name in kwargs:
            return kwargs[name]
        return ''

    def get_arg(self, name, *args):
        if len(args) > 0:
            return args[0]
        return ''

    def execute(self, required_parameter, *args):
        raise NotImplementedError('execute is not implemented')


from provider import Provider

class AbstractFactory():
    def __init__(self):
        self.factories = {}

    def set(self, factory):
        instance = factory()

        if not hasattr(instance, 'interface_type'):
            raise ValueError('factory requires provider/interface type')

        self.factories[instance.interface_type] = factory

    def remove(self, factory):
        self.factories.remove(factory)

    def resolve(self, provider, *args, **kwargs):
        interface_type = type(provider)

        if interface_type not in self.factories:
            return None

        factory = self.factories[interface_type]

        if factory is None:
            return None

        return factory().create(*args, **kwargs) 


from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

try:
   import pypandoc
   long_description = pypandoc.convert('README.md', 'rst')
   with open('README.rst', 'w+') as f:
       f.write(long_description)       
except (IOError, ImportError):
   long_description = ''

from distutils.core import setup
setup(
    name = 'abstractfactory',
    packages = ['project'],
    version = '1.0.1',
    description = 'Abstract Factory functionality for Python',
    long_description=long_description,
    author = 'David Betz',
    author_email = 'dfb@davidbetz.net',
    url = 'https://github.com/davidbetz/pyabstractfactory',
    keywords = ['abstract-factory', 'patterns'],
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ],
)


import unittest

from abstract_factory import AbstractFactory

from mock import MockProviderFactory, MockProvider

class TestAbstractFactory(unittest.TestCase):
    def test_bad(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider, "bad") 
        try:
            result = provider.execute("nope")
        except NotImplementedError:
            pass
        else:
            self.fail('NotImplementedError not raised')

    def test_does_not_exist(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        try:
            provider = abstractFactory.resolve(MockProvider, "does not exist") 
        except ValueError:
            pass
        else:
            self.fail('ValueError not raised')

    def test_execute_no_parameter(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider) 
        result = provider.execute() 

        self.assertEqual(result, "mock provider") 

    def test_execute_with_parameter(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider) 
        result = provider.execute("hello") 

        self.assertEqual(result, "hellomock provider") 

    def test_with_override_without_parameter(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider, "alt") 
        result = provider.execute() 

        self.assertEqual(result, "alternative mock provider") 

    def test_with_override_and_parameter(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider, "alt") 
        result = provider.execute("hi") 

        self.assertEqual(result, "hialternative mock provider") 

    def test_with_override_and_kwargs(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider, "alt", taco="keyword param") 
        result = provider.execute("hi")
        
        self.assertEqual(result, "hikeyword paramalternative mock provider") 

    def test_with_override_and_kwargs_and_paramkwargs(self):
        abstractFactory = AbstractFactory()
        abstractFactory.set(MockProviderFactory)
        provider = abstractFactory.resolve(MockProvider, "alt", taco="keyword param") 
        result = provider.execute("hi", burrito="parameter keyword param") 

        self.assertEqual(result, "hikeyword paramalternative mock providerparameter keyword param") 

if __name__ == '__main__':
    unittest.main()



from abstract_factory import AbstractFactory, Provider

class MockProvider():
    def execute(self, *args, **kwargs):
        raise NotImplementedError('execute is not implemented')

class BadMockProvider(Provider, MockProvider):
    def execute_spelled_wrong(self, *args, **kwargs):
        pass

class CoreMockProvider(Provider, MockProvider):
    def execute(self, *args, **kwargs):
        param = ''
        if len(args) > 0:
            param = args[0]

        return "{}mock provider".format(param)

class MockAlternativeProvider(Provider, MockProvider):
    def __init__(self, **kwargs):
        self.keyword_arguments = kwargs

    def execute(self, *args, **kwargs):
        arg = self.get_arg(0, *args)
        taco = self.get_kwarg('taco', **self.keyword_arguments)
        burrito = self.get_kwarg('burrito', **kwargs)
        result =  "{}{}alternative mock provider{}".format(arg, taco, burrito)
        return result

class MockProviderFactory():
    def __init__(self):
        self.interface_type = type(MockProvider)

    def create(self, *args, **kwargs):
        hint = None
        if len(args) > 0:
            hint = args[0].lower() 

        param = ''
        if len(args) > 1:
            param = args[1].lower() 

        example_provider_to_use_from_config_store = "mock" 

        provider = None

        name = (hint or example_provider_to_use_from_config_store or '').lower() 
        if name is None or len(name) == 0:
            return None 

        if name == "mock":
            provider = CoreMockProvider() 
        elif name == "alt":
            provider = MockAlternativeProvider(**kwargs) 
        elif name == "bad":
            provider = BadMockProvider()
        else:
            raise ValueError("no") 

        return provider 
